package de.fhdw.wtf.parser.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.exception.parser.NoArrowException;
import de.fhdw.wtf.common.exception.parser.NoBracketCloseException;
import de.fhdw.wtf.common.exception.parser.NoBracketOpenException;
import de.fhdw.wtf.common.exception.parser.NoColonException;
import de.fhdw.wtf.common.exception.parser.NoCurlyBracketCloseException;
import de.fhdw.wtf.common.exception.parser.NoCurlyBracketOpenException;
import de.fhdw.wtf.common.exception.parser.NoDoubleSquareBracketCloseException;
import de.fhdw.wtf.common.exception.parser.NoEqualException;
import de.fhdw.wtf.common.exception.parser.NoGroupElementException;
import de.fhdw.wtf.common.exception.parser.NoPlusSymbolException;
import de.fhdw.wtf.common.exception.parser.NoSemicolonException;
import de.fhdw.wtf.common.exception.parser.NoSquareBracketCloseException;
import de.fhdw.wtf.common.exception.parser.NoSquareBracketOpenException;
import de.fhdw.wtf.common.exception.parser.NoTypeException;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.parser.OperationModifierAlreadyAddedException;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.ClassToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.symbols.ArrowToken;
import de.fhdw.wtf.common.token.symbols.BracketCloseToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CommaToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.DoubleSquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.DoubleSquareBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.EqualToken;
import de.fhdw.wtf.common.token.symbols.HyphenToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.parser.Parser;

/**
 * Tests the Exceptions which can be thrown by the parser.
 */
public class TestExceptions {
	
	/**
	 * {@link de.fhdw.wtf.common.token.TokenStream} is used for parser input.
	 */
	private TokenStream stream;
	
	/**
	 * {@link Parser} parses the given {@link de.fhdw.wtf.common.token.TokenStream} to a
	 * {@link de.fhdw.wtf.common.ast.Model}.
	 */
	private Parser parser;
	
	/**
	 * Represents: ":".
	 */
	private ColonToken colonToken;
	
	/**
	 * Represents: "group".
	 */
	private GroupToken groupToken;
	
	/**
	 * Represents: "=".
	 */
	private EqualToken equalToken;
	
	/**
	 * Represents: "[".
	 */
	private SquareBracketOpenToken squareBracketOpenToken;
	
	/**
	 * Represents: "]".
	 */
	private SquareBracketCloseToken squareBracketCloseToken;
	
	/**
	 * Represents: ";".
	 */
	private SemicolonToken semicolonToken;
	
	/**
	 * Represents: "class".
	 */
	private ClassToken classToken;
	
	/**
	 * Represents: "{".
	 */
	private CurlyBracketOpenToken curlyBracketOpenToken;
	
	/**
	 * Represents: "}".
	 */
	private CurlyBracketCloseToken curlyBracketCloseToken;
	
	/**
	 * Represents: "(".
	 */
	private BracketOpenToken bracketOpenToken;
	
	/**
	 * Represents: ")".
	 */
	private BracketCloseToken bracketCloseToken;
	
	/**
	 * Represents: "".
	 */
	private EndToken endToken;
	
	/**
	 * Represents: "->".
	 */
	private ArrowToken arrowToken;
	
	/**
	 * Represents: ",".
	 */
	private CommaToken commaToken;
	
	/**
	 * Represents: "abstract".
	 */
	private AbstractToken abstractToken;
	
	/**
	 * Represents: "[[".
	 */
	private DoubleSquareBracketOpenToken doubleSquareBracketOpenToken;
	
	/**
	 * Represents: "]]".
	 */
	private DoubleSquareBracketCloseToken doubleSquareBracketCloseToken;
	
	/**
	 * Creates the most important {@link de.fhdw.wtf.common.token.Token}s.
	 */
	@Before
	public void setUp() {
		this.colonToken = ColonToken.create(Position.create("", 1, 1, 0));
		this.groupToken = GroupToken.create(Position.create("", 1, 1, 0));
		this.equalToken = EqualToken.create(Position.create("", 1, 1, 0));
		this.squareBracketOpenToken = SquareBracketOpenToken.create(Position.create("", 1, 1, 0));
		this.squareBracketCloseToken = SquareBracketCloseToken.create(Position.create("", 1, 1, 0));
		this.bracketOpenToken = BracketOpenToken.create(Position.create("", 1, 1, 0));
		this.bracketCloseToken = BracketCloseToken.create(Position.create("", 1, 1, 0));
		this.semicolonToken = SemicolonToken.create(Position.create("", 1, 1, 0));
		this.classToken = ClassToken.create(Position.create("", 1, 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 1, 1, 0));
		this.curlyBracketCloseToken = CurlyBracketCloseToken.create(Position.create("", 1, 1, 0));
		this.endToken = EndToken.create(Position.create("", 1, 1, 0));
		this.abstractToken = AbstractToken.create(Position.create("", 1, 1, 0));
		this.commaToken = CommaToken.create(Position.create("", 1, 1, 0));
		this.doubleSquareBracketCloseToken = DoubleSquareBracketCloseToken.create(Position.create("", 1, 1, 0));
		this.doubleSquareBracketOpenToken = DoubleSquareBracketOpenToken.create(Position.create("", 1, 1, 0));
		this.arrowToken = ArrowToken.create(Position.create("", 1, 1, 0));
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[(MyParameter:String)-{}]];};];.
	 */
	@Test
	public void testNoArrowException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(HyphenToken.create(Position.createDummyPosition()));
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noArrowException = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoArrowException) {
					noArrowException = true;
				}
			}
			assertTrue(noArrowException);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:(name:);};];.
	 */
	@Test
	public void testNoTypeExceptionProduct() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken name = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(name);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noType = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoTypeException) {
					noType = true;
				}
			}
			assertTrue(noType);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:{String,};};];.
	 */
	@Test
	public void testNoTypeExceptionSum() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(string);
		this.stream.add(this.commaToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noType = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoTypeException) {
					noType = true;
				}
			}
			assertTrue(noType);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[(MyParameter:String)->{}]] abstract abstract;};];.
	 */
	@Test
	public void testOperationModifierAlreadyAddedException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean doubleOperationModifier = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof OperationModifierAlreadyAddedException) {
					doubleOperationModifier = true;
				}
			}
			assertTrue(doubleOperationModifier);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[(MyParameter:String->{}]];};];.
	 */
	@Test
	public void testNoBracketCloseException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noBracketClose = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoBracketCloseException) {
					noBracketClose = true;
				}
			}
			assertTrue(noBracketClose);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[(MyParameter:String)->{}]]};];.
	 */
	@Test
	public void testNoSemicolonExceptionOperation() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noSemicolon = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoSemicolonException) {
					noSemicolon = true;
				}
			}
			assertTrue(noSemicolon);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:(name:String;};];.
	 */
	@Test
	public void testNoBracketCloseExceptionAttribute() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken name = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(name);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noBracketClose = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoBracketCloseException) {
					noBracketClose = true;
				}
			}
			assertTrue(noBracketClose);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:(name:String)};];.
	 */
	@Test
	public void testNoSemicolonExceptionAttribute() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken name = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(name);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noSemicolon = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoSemicolonException) {
					noSemicolon = true;
				}
			}
			assertTrue(noSemicolon);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:(name:String);} MyClass2:class={};];.
	 */
	@Test
	public void testNoSemicolonExceptionClass() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myClass2 = IdentifierToken.create("MyClass2", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken name = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(name);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(myClass2);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noSemicolon = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoSemicolonException) {
					noSemicolon = true;
				}
			}
			assertTrue(noSemicolon);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[MyParameter:String)->{}]];};];.
	 */
	@Test
	public void testNoBracketOpenException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noBracketOpen = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoBracketOpenException) {
					noBracketOpen = true;
				}
			}
			assertTrue(noBracketOpen);
		}
	}
	
	/**
	 * MyGroup-group=[];.
	 */
	@Test
	public void testNoColonException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(HyphenToken.create(Position.createDummyPosition()));
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noColon = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoColonException) {
					noColon = true;
				}
			}
			assertTrue(noColon);
		}
	}
	
	/**
	 * MyGroup;group=[];.
	 */
	@Test
	public void testNoColonExceptionSemicolon() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noColon = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoColonException) {
					noColon = true;
				}
			}
			assertTrue(noColon);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[(MyParameter:String)->{}]];;];.
	 */
	@Test
	public void testNoCurlyBracketCloseExceptionClass() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noCurlyBracketClose = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoCurlyBracketCloseException) {
					noCurlyBracketClose = true;
				}
			}
			assertTrue(noCurlyBracketClose);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:{String, Integer;};];.
	 */
	@Test
	public void testNoCurlyBracketCloseExceptionSum() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken integer = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(string);
		this.stream.add(this.commaToken);
		this.stream.add(integer);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noCurlyBracketClose = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoCurlyBracketCloseException) {
					noCurlyBracketClose = true;
				}
			}
			assertTrue(noCurlyBracketClose);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class=MyOperation:[[(MyParameter:String)->{}]];};];.
	 */
	@Test
	public void testNoCurlyBracketOpenException1() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.doubleSquareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noCurlyBracketOpen = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoCurlyBracketOpenException) {
					noCurlyBracketOpen = true;
				}
			}
			assertTrue(noCurlyBracketOpen);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class=[];];.
	 */
	@Test
	public void testNoCurlyBracketOpenException2() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(2, this.parser.getExceptions().size());
			boolean noCurlyBracketOpen = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoCurlyBracketOpenException) {
					noCurlyBracketOpen = true;
				}
			}
			assertTrue(noCurlyBracketOpen);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class=};];.
	 */
	@Test
	public void testNoCurlyBracketOpenException3() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noCurlyBracketOpen = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoCurlyBracketOpenException) {
					noCurlyBracketOpen = true;
				}
			}
			assertTrue(noCurlyBracketOpen);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class=MySecoundClass{};];.
	 */
	@Test
	public void testNoPlusSymbolException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken mySecondClass = IdentifierToken.create("MySecondClass", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(mySecondClass);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noPlusSymbol = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoPlusSymbolException) {
					noPlusSymbol = true;
				}
			}
			assertTrue(noPlusSymbol);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyOperation:[[(MyParameter:String)->{};};];.
	 */
	@Test
	public void testDoubleSquareBracketCloseException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myOperation = IdentifierToken.create("MyOperation", Position.create("", 1, 1, 0));
		final IdentifierToken myParameter = IdentifierToken.create("MyParameter", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myOperation);
		this.stream.add(this.colonToken);
		this.stream.add(this.doubleSquareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(myParameter);
		this.stream.add(this.colonToken);
		this.stream.add(string);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noDoubleSquareBracketClose = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoDoubleSquareBracketCloseException) {
					noDoubleSquareBracketClose = true;
				}
			}
			assertTrue(noDoubleSquareBracketClose);
		}
	}
	
	/**
	 * MyGroup:group[MyClass:class={};];.
	 */
	@Test
	public void testNoEqualsExceptionGroup() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noEqual = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoEqualException) {
					noEqual = true;
				}
			}
			assertTrue(noEqual);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class{};];.
	 */
	@Test
	public void testNoEqualsExceptionClass() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noEqual = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoEqualException) {
					noEqual = true;
				}
			}
			assertTrue(noEqual);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:={};];.
	 */
	@Test
	public void testNoGroupElementException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noGroupElement = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoGroupElementException) {
					noGroupElement = true;
				}
			}
			assertTrue(noGroupElement);
		}
	}
	
	/**
	 * MyGroup:group=[MyClass:class={MyAttribute:[Integer->String;};];.
	 */
	@Test
	public void testNoSquareBracketCloseException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		final IdentifierToken myClass = IdentifierToken.create("MyClass", Position.create("", 1, 1, 0));
		final IdentifierToken myAttribute = IdentifierToken.create("MyAttribute", Position.create("", 1, 1, 0));
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken integer = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(myClass);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(myAttribute);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(integer);
		this.stream.add(this.arrowToken);
		this.stream.add(string);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
			boolean noSquareBracketClose = false;
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoSquareBracketCloseException) {
					noSquareBracketClose = true;
				}
			}
			assertTrue(noSquareBracketClose);
		}
	}
	
	/**
	 * MyGroup:group=];.
	 */
	@Test
	public void testNoSquareBracketOpenException() {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		final IdentifierToken myGroup = IdentifierToken.create("MyGroup", Position.create("", 1, 1, 0));
		this.stream.add(myGroup);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			boolean noSquareBracketOpen = false;
			assertEquals(1, this.parser.getExceptions().size());
			for (final Exception ex : this.parser.getExceptions()) {
				if (ex instanceof NoSquareBracketOpenException) {
					noSquareBracketOpen = true;
				}
			}
			assertTrue(noSquareBracketOpen);
		}
	}
}
