package de.fhdw.wtf.parser.test;

import java.util.Iterator;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.visitor.GroupElementReturnVisitor;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;
import de.fhdw.wtf.common.token.keywords.FindableToken;
import de.fhdw.wtf.common.token.symbols.ArrowToken;
import de.fhdw.wtf.common.token.symbols.AsteriskToken;
import de.fhdw.wtf.common.token.symbols.BracketCloseToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.PlusSymbolToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;

/**
 * Tests the positions of First- and LastToken.
 */
public class TestGetFirstAndLastToken extends TestCase {
	
	/**
	 * gruppenName:group=[]; .
	 * 
	 * @throws NoValidTokenStreamException
	 */
	@Test
	public void testSimpleGroup() throws NoValidTokenStreamException {
		final SimpleScannerInput input = new SimpleScannerInput("gruppenName:group=[];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		final Token expectedFirstToken = IdentifierToken.create("gruppenName", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 21 + 1, 21));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 21 + 1, 21));
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		final Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 11 + 1, 11));
		assertEquals(expectedLastToken, name.getLastToken());
	}
	
	/**
	 * gruppenName:group=[klassenName:class={};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClass() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("gruppenName:group=[klassenName:class={};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("gruppenName", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 42 + 1, 42));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 42 + 1, 42));
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 11 + 1, 11));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("klassenName", Position.create("", 0 + 1, 19 + 1, 19));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 40 + 1, 40));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		final UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("klassenName", Position.create("", 0 + 1, 19 + 1, 19));
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 30 + 1, 30));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// gruppenName>klassenName
		expectedFirstToken = IdentifierToken.create("klassenName", Position.create("", 0 + 1, 19 + 1, 19)); // bleibt
																											// der
																											// klassenname,
																											// nicht
																											// der
																											// gruppenname
																											// -
																											// in
																											// absprache
																											// mit
																											// CS
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 30 + 1, 30));
		assertEquals(expectedFirstToken, name.getFirstToken());
		assertEquals(expectedLastToken, name.getLastToken());
	}
	
	/**
	 * Group1:group=[Group2:group=[Group3:group=[];];]; .
	 * 
	 * @throws NoValidTokenStreamException
	 */
	@Test
	public void testThreeGroups() throws NoValidTokenStreamException {
		final SimpleScannerInput input = new SimpleScannerInput("Group1:group=[Group2:group=[Group3:group=[];];];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 48 + 1, 48));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 48 + 1, 48));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 6 + 1, 6));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Gruppe Group2
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final Group group2 = iterator2.next().accept(new GroupElementReturnVisitor<Group>() {
			
			@Override
			public Group handle(final Group group) {
				return group;
			}
			
			@Override
			public Group handle(final ClassType clss) {
				fail();
				return null;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Group2", Position.create("", 0 + 1, 14 + 1, 14));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 46 + 1, 46));
		assertEquals(expectedFirstToken, group2.getFirstToken());
		assertEquals(expectedLastToken, group2.getLastToken());
		
		// Gruppenname
		name = group2.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 20 + 1, 20));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Gruppe Group3
		final Iterator<GroupElement> iterator3 = group2.getGroupElements().iterator();
		final Group group3 = iterator3.next().accept(new GroupElementReturnVisitor<Group>() {
			
			@Override
			public Group handle(final Group group) {
				return group;
			}
			
			@Override
			public Group handle(final ClassType clss) {
				fail();
				return null;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Group3", Position.create("", 0 + 1, 28 + 1, 28));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 44 + 1, 44));
		assertEquals(expectedFirstToken, group3.getFirstToken());
		assertEquals(expectedLastToken, group3.getLastToken());
		
		// Gruppenname
		name = group3.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 34 + 1, 34));
		assertEquals(expectedLastToken, name.getLastToken());
		
	}
	
	/**
	 * Group1:group=[Superclass:class={};];Group2:group=[Class:class=Superclass+ {};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassWithHierarchy() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group1:group=[Superclass:class={};];Group2:group=[Class:class=Superclass+{};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 78 + 1, 78));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = IdentifierToken.create("Group2", Position.create("", 0 + 1, 36 + 1, 36));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 6 + 1, 6));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Gruppe Group2
		final Group group2 = iterator.next();
		
		expectedFirstToken = IdentifierToken.create("Group2", Position.create("", 0 + 1, 36 + 1, 36));
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 78 + 1, 78));
		
		assertEquals(expectedFirstToken, group2.getFirstToken());
		assertEquals(expectedLastToken, group2.getLastToken());
		
		// Gruppenname
		name = group2.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 42 + 1, 42));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse Superclass
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Superclass", Position.create("", 0 + 1, 14 + 1, 14));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 34 + 1, 34));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Superclass", Position.create("", 0 + 1, 14 + 1, 14));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 24 + 1, 24));
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Klasse Class
		final Iterator<GroupElement> iterator3 = group2.getGroupElements().iterator();
		final GroupElement groupElement2 = iterator3.next();
		final ClassType class2 = groupElement2.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 50 + 1, 50));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 76 + 1, 76));
		
		assertEquals(expectedFirstToken, class2.getFirstToken());
		assertEquals(expectedLastToken, class2.getLastToken());
		
		// Klassenname
		name = class2.getName();
		
		// klassenName
		lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 50 + 1, 50));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 55 + 1, 55));
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Vererbung Superclass
		final Iterator<Type> iterator4 = class2.getSuperTypes().iterator();
		final Type typeProxy = iterator4.next();
		expectedFirstToken = IdentifierToken.create("Superclass", Position.create("", 0 + 1, 62 + 1, 62));
		assertEquals(expectedFirstToken, typeProxy.getFirstToken());
		expectedLastToken = PlusSymbolToken.create(Position.create("", 0 + 1, 72 + 1, 72));
		assertEquals(expectedLastToken, typeProxy.getLastToken());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:String;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleAttributte() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute:String;};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 47 + 1, 47));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 47 + 1, 47));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 5 + 1, 5));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse Class
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 13 + 1, 13));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 45 + 1, 45));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		final UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 13 + 1, 13));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 18 + 1, 18));
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Attribut Attribute
		final Iterator<Attribute> iterator3 = class1.getAttributes().iterator();
		final Attribute attribute = iterator3.next();
		
		expectedFirstToken = IdentifierToken.create("Attribute", Position.create("", 0 + 1, 26 + 1, 26));
		expectedLastToken = CurlyBracketCloseToken.create(Position.create("", 0 + 1, 43 + 1, 43));
		
		assertEquals(expectedFirstToken, attribute.getFirstToken());
		assertEquals(expectedLastToken, attribute.getLastToken());
		
		// Attributtyp String
		final Type attrType = attribute.getAttrType();
		
		expectedFirstToken = IdentifierToken.create("String", Position.create("", 0 + 1, 36 + 1, 36));
		expectedLastToken = SemicolonToken.create(Position.create("", 0 + 1, 42 + 1, 42));
		
		assertEquals(expectedFirstToken, attrType.getFirstToken());
		assertEquals(expectedLastToken, attrType.getLastToken());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:Integer*;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testList() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute:Integer*;};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 50, 49));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 50, 49));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 5 + 1, 5));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse Class
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 13 + 1, 13));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 48, 47));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		final UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 13 + 1, 13));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 18 + 1, 18));
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Attribut Attribute
		final Iterator<Attribute> iterator3 = class1.getAttributes().iterator();
		final Attribute attribute = iterator3.next();
		
		expectedFirstToken = IdentifierToken.create("Attribute", Position.create("", 0 + 1, 26 + 1, 26));
		expectedLastToken = CurlyBracketCloseToken.create(Position.create("", 0 + 1, 46, 45));
		
		assertEquals(expectedFirstToken, attribute.getFirstToken());
		assertEquals(expectedLastToken, attribute.getLastToken());
		
		// Attributtyp List
		final Type attrType = attribute.getAttrType();
		
		expectedFirstToken = IdentifierToken.create("Integer", Position.create("", 0 + 1, 36 + 1, 36));
		expectedLastToken = SemicolonToken.create(Position.create("", 0 + 1, 45, 44));
		
		assertEquals(expectedFirstToken, attrType.getFirstToken());
		assertEquals(expectedLastToken, attrType.getLastToken());
		
		// List of
		final ListType list = (ListType) attrType;
		
		expectedFirstToken = IdentifierToken.create("Integer", Position.create("", 0 + 1, 37, 36));
		expectedLastToken = SemicolonToken.create(Position.create("", 0 + 1, 45, 44));
		
		assertEquals(expectedFirstToken, list.getFirstToken());
		assertEquals(expectedLastToken, list.getLastToken());
		
		expectedLastToken = AsteriskToken.create(Position.create("", 0 + 1, 44, 43));
		
		assertEquals(expectedFirstToken, list.getOf().getFirstToken());
		assertEquals(expectedLastToken, list.getOf().getLastToken());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[String->Integer]**;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testListOfListOfMap() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute:[String->Integer]**;};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 61, 60));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 61, 60));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 5 + 1, 5));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse Class
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 1, 14, 13));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 1, 59, 58));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		final UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 1, 14, 13));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 1, 19, 18));
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Attribut Attribute
		final Iterator<Attribute> iterator3 = class1.getAttributes().iterator();
		final Attribute attribute = iterator3.next();
		
		expectedFirstToken = IdentifierToken.create("Attribute", Position.create("", 1, 27, 26));
		expectedLastToken = CurlyBracketCloseToken.create(Position.create("", 1, 57, 56));
		
		assertEquals(expectedFirstToken, attribute.getFirstToken());
		assertEquals(expectedLastToken, attribute.getLastToken());
		
		// Attributtyp List of List
		final ListType list = (ListType) attribute.getAttrType();
		
		expectedFirstToken = SquareBracketOpenToken.create(Position.create("", 1, 37, 36));
		expectedLastToken = SemicolonToken.create(Position.create("", 1, 56, 55));
		
		assertEquals(expectedFirstToken, list.getFirstToken());
		assertEquals(expectedLastToken, list.getLastToken());
		
		// List of Map
		final ListType list2 = (ListType) list.getOf();
		
		expectedFirstToken = SquareBracketOpenToken.create(Position.create("", 1, 37, 36));
		expectedLastToken = AsteriskToken.create(Position.create("", 1, 55, 54));
		
		assertEquals(expectedFirstToken, list2.getFirstToken());
		assertEquals(expectedLastToken, list2.getLastToken());
		
		// Map
		final MapType map = (MapType) list2.getOf();
		
		expectedFirstToken = SquareBracketOpenToken.create(Position.create("", 1, 37, 36));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 1, 53, 52));
		
		assertEquals(expectedFirstToken, map.getFirstToken());
		assertEquals(expectedLastToken, map.getLastToken());
	}
	
	/**
	 * Group:group=[Class:class={nummern:Integer* findable;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testAttributeWithModifier() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={nummern:Integer* findable;};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 0 + 1, 57, 56));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = EndToken.create(Position.create("", 0 + 1, 57, 56));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 5 + 1, 5));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse Class
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 13 + 1, 13));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 55, 54));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		final UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 13 + 1, 13));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		expectedLastToken = ColonToken.create(Position.create("", 0 + 1, 18 + 1, 18));
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Attribut Attribute
		final Iterator<Attribute> iterator3 = class1.getAttributes().iterator();
		final Attribute attribute = iterator3.next();
		
		expectedFirstToken = IdentifierToken.create("nummern", Position.create("", 0 + 1, 26 + 1, 26));
		expectedLastToken = CurlyBracketCloseToken.create(Position.create("", 0 + 1, 53, 52));
		
		assertEquals(expectedFirstToken, attribute.getFirstToken());
		assertEquals(expectedLastToken, attribute.getLastToken());
		
		// Modfifier
		final AttributeModifier modifier = attribute.getModifiers().iterator().next();
		expectedFirstToken = FindableToken.create(Position.create("", 0 + 1, 44, 43));
		expectedLastToken = SemicolonToken.create(Position.create("", 0 + 1, 52, 51));
		
		assertEquals(expectedFirstToken, modifier.getFirstToken());
		assertEquals(expectedLastToken, modifier.getLastToken());
		
		// Attributtyp List of Integer
		final ListType list = (ListType) attribute.getAttrType();
		
		expectedFirstToken = IdentifierToken.create("Integer", Position.create("", 0 + 1, 35, 34));
		expectedLastToken = FindableToken.create(Position.create("", 0 + 1, 44, 43));
		
		assertEquals(expectedFirstToken, list.getFirstToken());
		assertEquals(expectedLastToken, list.getLastToken());
		
		// Integer
		final Type type = list.getOf();
		
		expectedFirstToken = IdentifierToken.create("Integer", Position.create("", 0 + 1, 35, 34));
		expectedLastToken = AsteriskToken.create(Position.create("", 0 + 1, 42, 41));
		
		assertEquals(expectedFirstToken, type.getFirstToken());
		assertEquals(expectedLastToken, type.getLastToken());
	}
	
	/**
	 * Group:group=[Class:class={operation:[[(Number:Integer) -> String]] abstract;};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testOperation() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Class:class={operation:[[(Number:Integer) -> String]] abstract;};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser parser = Parser.create(scannerOutput);
		final Model result = parser.parse();
		
		// Model
		Token expectedFirstToken = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		Token expectedLastToken = EndToken.create(Position.create("", 1, 81, 80));
		
		assertEquals(expectedFirstToken, result.getFirstToken());
		assertEquals(expectedLastToken, result.getLastToken());
		
		// Gruppe Group1
		final Iterator<Group> iterator = result.getGroups().iterator();
		final Group group = iterator.next();
		
		expectedLastToken = EndToken.create(Position.create("", 1, 81, 80));
		
		assertEquals(expectedFirstToken, group.getFirstToken());
		assertEquals(expectedLastToken, group.getLastToken());
		
		// Gruppenname
		Name name = group.getName();
		assertEquals(expectedFirstToken, name.getFirstToken());
		
		expectedLastToken = ColonToken.create(Position.create("", 1, 6, 5));
		assertEquals(expectedLastToken, name.getLastToken());
		
		// Klasse Class
		final Iterator<GroupElement> iterator2 = group.getGroupElements().iterator();
		final GroupElement groupElement = iterator2.next();
		final ClassType class1 = groupElement.accept(new GroupElementReturnVisitor<ClassType>() {
			
			@Override
			public ClassType handle(final Group group) {
				fail();
				return null;
			}
			
			@Override
			public ClassType handle(final ClassType clss) {
				return clss;
			}
		});
		
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 1, 14, 13));
		expectedLastToken = SquareBracketCloseToken.create(Position.create("", 1, 79, 78));
		
		assertEquals(expectedFirstToken, class1.getFirstToken());
		assertEquals(expectedLastToken, class1.getLastToken());
		
		// Klassenname
		name = class1.getName();
		
		// klassenName
		final UnqualifiedName lastAddedName = name.getLastAddedName();
		expectedFirstToken = IdentifierToken.create("Class", Position.create("", 1, 14, 13));
		expectedLastToken = ColonToken.create(Position.create("", 1, 19, 18));
		assertEquals(expectedFirstToken, lastAddedName.getFirstToken());
		assertEquals(expectedLastToken, lastAddedName.getLastToken());
		
		// Operation
		final Iterator<Operation> iterator3 = class1.getOperations().iterator();
		final Operation operation = iterator3.next();
		
		expectedFirstToken = IdentifierToken.create("operation", Position.create("", 1, 27, 26));
		expectedLastToken = CurlyBracketCloseToken.create(Position.create("", 1, 77, 76));
		
		assertEquals(expectedFirstToken, operation.getFirstToken());
		assertEquals(expectedLastToken, operation.getLastToken());
		
		// Input-Parameter
		final ProductType params = operation.getParameters();
		
		final Iterator<ProductElementType> iterator4 = params.getElements().iterator();
		final ProductElementType parameterIdentifier = iterator4.next();
		
		expectedFirstToken = IdentifierToken.create("Number", Position.create("", 1, 40, 39));
		expectedLastToken = BracketCloseToken.create(Position.create("", 1, 54, 53));
		assertEquals(expectedFirstToken, parameterIdentifier.getFirstToken());
		assertEquals(expectedLastToken, parameterIdentifier.getLastToken());
		
		expectedFirstToken = BracketOpenToken.create(Position.create("", 1, 39, 38));
		expectedLastToken = ArrowToken.create(Position.create("", 1, 56, 55));
		assertEquals(expectedFirstToken, params.getFirstToken());
		assertEquals(expectedLastToken, params.getLastToken());
	}
}
