package de.fhdw.wtf.walker.tasks.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Iterator;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions;
import de.fhdw.wtf.tooling.SyntaxCheck;

/**
 * This test case tests the function of AnalyzeInheritanceTreesTask and check the generated map of this task and the
 * elements.
 */
public class TestAnalyzeInheritanceTreesTask {
	
	/**
	 * Tests an empty group. Model: group1:group = [];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testEmptyGroup() throws Exception {
		final String input = "group1:group = [];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			fail();
		}
		assertTrue(true);
	}
	
	/**
	 * Tests an empty class. Model: group1:group = [A:class = {};]; It is necessary to define one or more constructors
	 * and in the model no constructors exist.
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testEmptyClass() throws Exception {
		final String input = "group1:group = [A:class = {};];";
		
		try {
			SyntaxCheck.getInstance().getModelFromString(input);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertTrue(true);
			// Konstruktoren fehlen daher fliegt diese
		}
	}
	
	/**
	 * Tests two empty classes. Model: group1:group = [A:class = {}; B:class = {};]; It is necessary to define one or
	 * more constructors and in the model no constructors exist.
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testTwoEmptyClass() throws Exception {
		final String input = "group1:group = [A:class = {}; B:class = {};];";
		
		try {
			SyntaxCheck.getInstance().getModelFromString(input);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertTrue(true);
			// Konstruktoren fehlen daher fliegt diese
		}
	}
	
	/**
	 * Tests one class with one constructor. Model: group1:group = [A:class = {();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testOneClassOneConstructor() throws Exception {
		final String input = "group1:group = [A:class = {();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertTrue(model.getConstructorCallDependencies().get(current).isEmpty());
				break;
			default:
				fail();
				break;
			}
		}
		assertTrue(true);
	}
	
	/**
	 * Tests two classes both with one constructor. Model: group1:group = [A:class = {();}; B:class = {();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testTwoClassesBothWithOneConstructor() throws Exception {
		final String input = "group1:group = [A:class = {();}; B:class = {();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertTrue(model.getConstructorCallDependencies().get(current).isEmpty());
				break;
			case "group1>B":
				assertTrue(true);
				assertTrue(model.getConstructorCallDependencies().get(current).isEmpty());
				break;
			default:
				fail();
				break;
			}
		}
		assertTrue(true);
	}
	
	/**
	 * Tests the simple inheritance between two classes that the second class inherits the first class. Both classes
	 * have no constructor.
	 * 
	 * Model: group1:group = [A:class = {}; B:class = A + {};];
	 * 
	 * It is necessary to define one or more constructors and in the model no constructors exist.
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testSimpleInheritance() throws Exception {
		final String input = "group1:group = [A:class = {}; B:class = A + {};];";
		
		try {
			SyntaxCheck.getInstance().getModelFromString(input);
			fail();
		} catch (final MultipleCheckExceptions e) {
			assertTrue(true);
			// Konstruktoren fehlen daher fliegt diese
		}
	}
	
	/**
	 * Tests the simple inheritance between two classes that the second class inherits the first class. Both classes
	 * have one default constructor.
	 * 
	 * Model: group1:group = [A:class = {();}; B:class = A + {() = A();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testSimpleInheritanceWithConstructors() throws Exception {
		final String input = "group1:group = [A:class = {();}; B:class = A + {() = A();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertTrue(model.getConstructorCallDependencies().get(current).isEmpty());
				break;
			case "group1>B":
				assertTrue(true);
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
		assertTrue(true);
	}
	
	/**
	 * Tests the model where a class implements two other classes.
	 * 
	 * Model: group1:group = [A :class={();}; B:class = {();}; C:class= A + B + {()=A() + B();};];
	 * 
	 * @throws Exception
	 *             x
	 * 
	 */
	@Test
	public void testInheritanceOfTwoSuperclasses() throws Exception {
		final String input = "group1:group = [A :class={();}; B:class = {();}; C:class= A + B + {()=A() + B();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals("group1>B", cIterator.next().toString());
				assertEquals(2, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the model where a class implements three other classes.
	 * 
	 * Model: group1:group = [A :class={();}; B:class = {();}; C:class= {();}; D:class= A + B + C + {()= A() + B() +
	 * C();};];
	 * 
	 * @throws Exception
	 *             x
	 * 
	 */
	@Test
	public void testInheritanceOfThereSuperclasses() throws Exception {
		final String input =
				"group1:group = [A :class={();}; B:class = {();}; C:class= {();};"
						+ " D:class= A + B + C + {()= A() + B() + C();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> dIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", dIterator.next().toString());
				assertEquals("group1>B", dIterator.next().toString());
				assertEquals("group1>C", dIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the Diamond.
	 * 
	 * Model: group1:group = [A :class={();}; B:class= A + {()=A();}; C:class= A + {()=A();}; D:class= B + C
	 * +{()=A()+B()+C();};];
	 * 
	 * @throws Exception
	 *             x
	 * 
	 */
	@Test
	public void testDiamond() throws Exception {
		final String input =
				"group1:group = [A :class={();}; B:class= A + {()=A();}; C:class= A + {()=A();};"
						+ " D:class= B + C +{()=A()+B()+C();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> aIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", aIterator.next().toString());
				assertEquals("group1>B", aIterator.next().toString());
				assertEquals("group1>C", aIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the Diamond changed the order of the superTypes.
	 * 
	 * Model: group1:group = [A :class={();}; B:class= A + {()=A();}; C:class= A + {()=A();}; D:class= C + B
	 * +{()=A()+B()+C();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testDiamondWithChangedOrderInInheritance() throws Exception {
		final String input =
				"group1:group = [A :class={();}; B:class= A + {()=A();}; C:class= A + {()=A();};"
						+ " D:class= C + B + {()=A()+B()+C();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> aIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", aIterator.next().toString());
				assertEquals("group1>B", aIterator.next().toString());
				assertEquals("group1>C", aIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the Diamond changed the order of the superConstructor-calls.
	 * 
	 * Model: group1:group = [A :class={();}; B:class= A + {()=A();}; C:class= A + {()=A();}; D:class= B + C
	 * +{()=A()+C()+B();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testDiamondWithChangedOrderInSuperconstructorCall() throws Exception {
		final String input =
				"group1:group = [A :class={();}; B:class= A + {()=A();}; C:class= A + {()=A();};"
						+ " D:class= C + B + {()=A()+C()+B();};];";
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> aIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", aIterator.next().toString());
				assertEquals("group1>B", aIterator.next().toString());
				assertEquals("group1>C", aIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the Diamond under a diamond.
	 * 
	 * Model: group1:group = [A:class={();}; B:class = A + {()=A();}; C:class = A + {()=A();}; D:class = B + C + {()=A()
	 * + B() + C();}; E:class = D + {()=D();}; F:class = D + {()=D();}; G:class = E + F + {()=D() + E() + F();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testDiamondUnderDiamond() throws Exception {
		final String input =
				"group1:group = [A:class={();}; B:class = A + {()=A();}; C:class = A + {()=A();};"
						+ " D:class = B + C + {()=A() + B() + C();}; E:class = D + {()=D();}; F:class = D + {()=D();};"
						+ " G:class = E + F + {()=D() + E() + F();};];";
		
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> dIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", dIterator.next().toString());
				assertEquals("group1>B", dIterator.next().toString());
				assertEquals("group1>C", dIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>E":
				final Iterator<Type> eIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", eIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>F":
				final Iterator<Type> fIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", fIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>G":
				final Iterator<Type> gIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", gIterator.next().toString());
				assertEquals("group1>E", gIterator.next().toString());
				assertEquals("group1>F", gIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the Diamond under a diamond with hands.
	 * 
	 * Model: group1:group = [A:class={();}; B:class = A + {()=A();}; C:class = A + {()=A();}; D:class = B + C + {()=A()
	 * + B() + C();}; E:class = D + H {()=D() + H();}; F:class = D + I + {()=D() + I();}; G:class = E + F + {()=D() +
	 * E() + F();}; H:class = {();}; I:class = {();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testDiamondUnderDiamondWithHandyCalledCactus() throws Exception {
		final String input =
				"group1:group = [A:class={();}; B:class = A + {()=A();}; C:class = A + {()=A();};"
						+ " D:class = B + C + {()=A() + B() + C();}; E:class = D + H + {()=D() + H();};"
						+ " F:class = D + I + {()=D() + I();}; G:class = E + F + {()=D() + E() + F();};"
						+ " H:class = {();}; I:class = {();};];";
		
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> dIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", dIterator.next().toString());
				assertEquals("group1>B", dIterator.next().toString());
				assertEquals("group1>C", dIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>E":
				final Iterator<Type> eIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", eIterator.next().toString());
				assertEquals("group1>H", eIterator.next().toString());
				assertEquals(2, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>F":
				final Iterator<Type> fIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", fIterator.next().toString());
				assertEquals("group1>I", fIterator.next().toString());
				assertEquals(2, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>G":
				final Iterator<Type> gIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", gIterator.next().toString());
				assertEquals("group1>E", gIterator.next().toString());
				assertEquals("group1>F", gIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>H":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>I":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the Diamond under a diamond with an extra path from the bottom to the top.
	 * 
	 * Model: group1:group = [A:class={();}; B:class = A + {()=A();}; C:class = A + {()=A();}; D:class = B + C + {()=A()
	 * + B() + C();}; E:class = D + {()=D();}; F:class = D + {()=D();}; G:class = E + F + H + {()=A() + H() + D() + E()
	 * + F();}; H:class = A + {() = A();};];
	 * 
	 * @throws Exception
	 *             x InvalidConstructorReferencedListException
	 */
	@Test
	public void testDiamondExtra() throws Exception {
		final String input =
				"group1:group = [A:class={();}; B:class = A + {()=A();}; C:class = A + {()=A();};"
						+ " D:class = B + C + {()=A() + B() + C();}; E:class = D + {()=D();}; F:class = D + {()=D();};"
						+ " G:class = E + F + H + {()= A() + H() + D() + E() + F();}; H:class = A + {() = A();};];";
		
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> dIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", dIterator.next().toString());
				assertEquals("group1>B", dIterator.next().toString());
				assertEquals("group1>C", dIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>E":
				final Iterator<Type> eIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", eIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>F":
				final Iterator<Type> fIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>D", fIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>G":
				final Iterator<Type> gIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", gIterator.next().toString());
				assertEquals("group1>D", gIterator.next().toString());
				assertEquals("group1>E", gIterator.next().toString());
				assertEquals("group1>F", gIterator.next().toString());
				assertEquals("group1>H", gIterator.next().toString());
				assertEquals(5, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>H":
				final Iterator<Type> hIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", hIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the large Diamond.
	 * 
	 * Model: group1:group = [A:class={();};B:class=
	 * A+{()=A();};C:class=A+{()=A();};D:class=A+{()=A();};E:class=A+{()=A(
	 * );};F:class=B+C+D+E+{()=A()+B()+C()+D()+E();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testLargeDiamond() throws Exception {
		final String input =
				"group1:group = [A:class={();};B:class= A+{()=A();};C:class=A+{()=A();};"
						+ "D:class=A+{()=A();};E:class=A+{()=A();};F:class=B+C+D+E+{()=A()+B()+C()+D()+E();};];";
		
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> dIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", dIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>E":
				final Iterator<Type> eIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", eIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>F":
				final Iterator<Type> fIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", fIterator.next().toString());
				assertEquals("group1>B", fIterator.next().toString());
				assertEquals("group1>C", fIterator.next().toString());
				assertEquals("group1>D", fIterator.next().toString());
				assertEquals("group1>E", fIterator.next().toString());
				assertEquals(5, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
	
	/**
	 * Tests the parallel Diamond.
	 * 
	 * Model: group1:group = [A:class={();};B:class=
	 * A+{()=A();};C:class=A+{()=A();};D:class=B+{()=B();};E:class=B+{()=B();};H:class=D+E+{()=B()+D()+E();};
	 * F:class=C+{()=C();};G:class=C+{()=C();};I:class=F+G+{()=C()+F()+G();};J:class=H+I+{()=A()+H()+I();};];
	 * 
	 * @throws Exception
	 *             x
	 */
	@Test
	public void testParallelDiamond() throws Exception {
		final String input =
				"group1:group = [A:class={();};B:class= A+{()=A();};C:class=A+{()=A();};"
						+ "D:class=B+{()=B();};E:class=B+{()=B();};H:class=D+E+{()=B()+D()+E();};"
						+ "F:class=C+{()=C();};G:class=C+{()=C();};I:class=F+G+{()=C()+F()+G();};"
						+ "J:class=H+I+{()=A()+H()+I();};];";
		
		final Model model = SyntaxCheck.getInstance().getModelFromString(input);
		
		final Iterator<Type> iterator = model.getConstructorCallDependencies().keySet().iterator();
		while (iterator.hasNext()) {
			final Type current = iterator.next();
			
			switch (current.getTypeString()) {
			case "group1>A":
				assertTrue(true);
				assertEquals(0, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>B":
				final Iterator<Type> bIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", bIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>C":
				final Iterator<Type> cIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", cIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>D":
				final Iterator<Type> dIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>B", dIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>E":
				final Iterator<Type> eIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>B", eIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>F":
				final Iterator<Type> fIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>C", fIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>G":
				final Iterator<Type> gIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>C", gIterator.next().toString());
				assertEquals(1, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>H":
				final Iterator<Type> hIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>B", hIterator.next().toString());
				assertEquals("group1>D", hIterator.next().toString());
				assertEquals("group1>E", hIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>I":
				final Iterator<Type> iIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>C", iIterator.next().toString());
				assertEquals("group1>F", iIterator.next().toString());
				assertEquals("group1>G", iIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			case "group1>J":
				final Iterator<Type> jIterator = model.getConstructorCallDependencies().get(current).iterator();
				assertEquals("group1>A", jIterator.next().toString());
				assertEquals("group1>H", jIterator.next().toString());
				assertEquals("group1>I", jIterator.next().toString());
				assertEquals(3, model.getConstructorCallDependencies().get(current).size());
				break;
			default:
				fail();
				break;
			}
		}
	}
}
