package de.fhdw.wtf.walker.tasks.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.exception.walker.CyclicInheritanceException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.CyclicInheritanceCheck;
import de.fhdw.wtf.walker.tasks.TypeReferencer;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

public class TestCyclicInheritanceCheck extends TestCase {
	
	@Test
	public void testInheritFromItself() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=Person+{name:String;} abstract; "
						+ "NaturalPerson:class=Person+{operation:[[(Number:Integer) -> String]] abstract;}; "
						+ "CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 33, 32), ex.getStartPosition());
			assertEquals(Position.create("", 1, 39, 38), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testSameName() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=group2>Person+{name:String;} abstract; "
						+ "NaturalPerson:class=Person+{operation:[[(Number:Integer) -> String]] abstract;}; "
						+ "CorporateBody:class=Person+{}; group2:group=[Person:class={};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	@Test
	public void testInheritFromBasetype() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=String+{name:String;} abstract; "
						+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	@Test
	public void testCyclicInheritanceOverMoreClasses() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=NaturalPerson+{name:String;} abstract; "
						+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 92, 91), ex.getStartPosition());
			assertEquals(Position.create("", 1, 98, 97), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testInvalidTypeReference() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=NaturalPerson2+{name:String;} abstract; "
						+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			fail();
		}
	}
	
	@Test
	public void testWithoutTypeReferencer() throws InterruptedException, NoValidTokenStreamException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[Person:class={name:String;} abstract; NaturalPerson:class=Person+{}; "
								+ "CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		CyclicInheritanceCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		try {
			taskmanager.getResultsAndShutdown();
		} catch (final ExecutionException e) {
			// Erwartet, weil erst der TypeReferencer drüber laufen muss
			return;
		}
		fail();
	}
	
	@Test
	public void testCyclicInheritanceOverMoreClassesInFirstSupertype() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[Objekt:class={}; Person:class=NaturalPerson+Objekt+{name:String;} abstract; "
								+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 116, 115), ex.getStartPosition());
			assertEquals(Position.create("", 1, 122, 121), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testCyclicInheritanceOverMoreClassesInSecondSupertype() throws InterruptedException,
			ExecutionException, NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[Objekt:class={}; Person:class=Objekt+NaturalPerson+{name:String;} abstract; "
								+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 116, 115), ex.getStartPosition());
			assertEquals(Position.create("", 1, 122, 121), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testCyclicInheritanceOverMoreClassesBecauseOfTheSupertypeSupertype() throws InterruptedException,
			ExecutionException, NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[Objekt:class=NaturalPerson+{}; Person:class=Objekt+{name:String;} abstract; "
								+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 64, 63), ex.getStartPosition());
			assertEquals(Position.create("", 1, 70, 69), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testCyclicInheritanceOverMoreClassesInBothSupertypes() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[Person:class=NaturalPerson+CorporateBody+{name:String;} abstract; "
								+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 106, 105), ex.getStartPosition());
			assertEquals(Position.create("", 1, 112, 111), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testInheritanceFromTheSameClassTwice() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[Objekt:class={}; Person:class=Objekt+Objekt+{name:String;} abstract; "
								+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	@Test
	public void testInheritanceFromTwoClassesInTheSameInheritanceStructur() throws InterruptedException,
			ExecutionException, NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"personGroup:group=[XPerson:class=Person+NaturalPerson+{}; Person:class={name:String;} abstract; "
								+ "NaturalPerson:class=Person+{}; CorporateBody:class=Person+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	@Test
	public void testOverGroups() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("personGroup:group=[Person:class=group2>Person+{name:String;} abstract; "
						+ "NaturalPerson:class=Person+{operation:[[(Number:Integer) -> String]] abstract;}; "
						+ "CorporateBody:class=Person+{}; group2:group=[Person:class=NaturalPerson+{};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof CyclicInheritanceException) {
			final CyclicInheritanceException ex = (CyclicInheritanceException) exception;
			assertEquals(Position.create("", 1, 92, 91), ex.getStartPosition());
			assertEquals(Position.create("", 1, 98, 97), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	@Test
	public void testDiamondWithOutCyclus() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Gruppe:group=[D:class=B+C+{}; B:class=A+{}; C:class=A+{} ;A:class={};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(2, okResult.size());
		assertEquals(0, failResult.size());
	}
}
